@@ -623,16 +623,22 @@ static int p54spi_probe(struct spi_device *spi)
 	gpio_direction_input(p54spi_gpio_irq);
 
 	ret = request_irq(gpio_to_irq(p54spi_gpio_irq),
+#ifndef IRQF_NO_AUTOEN
 			  p54spi_interrupt, 0, "p54spi",
+#else
+			  p54spi_interrupt, IRQF_NO_AUTOEN, "p54spi",
+#endif
 			  priv->spi);
 	if (ret < 0) {
 		dev_err(&priv->spi->dev, "request_irq() failed");
 		goto err_free_gpio_irq;
 	}
 
-	irq_set_irq_type(gpio_to_irq(p54spi_gpio_irq), IRQ_TYPE_EDGE_RISING);
-
+#ifndef IRQF_NO_AUTOEN
 	disable_irq(gpio_to_irq(p54spi_gpio_irq));
+#endif
+
+	irq_set_irq_type(gpio_to_irq(p54spi_gpio_irq), IRQ_TYPE_EDGE_RISING);
 
 	INIT_WORK(&priv->work, p54spi_work);
 	init_completion(&priv->fw_comp);
@@ -670,7 +676,7 @@ err_free:
 	return ret;
 }
 
-static int p54spi_remove(struct spi_device *spi)
+static void p54spi_remove(struct spi_device *spi)
 {
 	struct p54s_priv *priv = spi_get_drvdata(spi);
 
@@ -685,9 +691,14 @@ static int p54spi_remove(struct spi_device *spi)
 	mutex_destroy(&priv->mutex);
 
 	p54_free_common(priv->hw);
+}
+#if LINUX_VERSION_IS_LESS(5,18,0)
+static int bp_p54spi_remove(struct spi_device *spi){
+	p54spi_remove(spi);
 
 	return 0;
 }
+#endif
 
 
 static struct spi_driver p54spi_driver = {
@@ -696,7 +707,12 @@ static struct spi_driver p54spi_driver = {
 	},
 
 	.probe		= p54spi_probe,
+#if LINUX_VERSION_IS_GEQ(5,18,0)
 	.remove		= p54spi_remove,
+#else
+	.remove = bp_p54spi_remove,
+#endif
+
 };
 
 module_spi_driver(p54spi_driver);
