@@ -188,7 +188,7 @@ static int qtnf_netdev_set_mac_address(struct net_device *ndev, void *addr)
 					     sa->sa_data);
 
 	if (ret)
-		memcpy(ndev->dev_addr, old_addr, ETH_ALEN);
+		eth_hw_addr_set(ndev, old_addr);
 
 	return ret;
 }
@@ -229,13 +229,8 @@ const struct net_device_ops qtnf_netdev_ops = {
 #else
 	.ndo_tx_timeout = bp_qtnf_netdev_tx_timeout,
 #endif
-
-#if LINUX_VERSION_IS_GEQ(4,11,0)
-	.ndo_get_stats64 = dev_get_tstats64,
-#else
-	.ndo_get_stats64 = bp_dev_get_tstats64,
-#endif
 	
+	.ndo_get_stats64 = dev_get_tstats64,
 	.ndo_set_mac_address = qtnf_netdev_set_mac_address,
 	.ndo_get_port_parent_id = qtnf_netdev_port_parent_id,
 };
@@ -494,10 +489,10 @@ int qtnf_core_net_attach(struct qtnf_wmac *mac, struct qtnf_vif *vif,
 	vif->netdev = dev;
 
 	dev->netdev_ops = &qtnf_netdev_ops;
-	netdev_set_def_destructor(dev);
+	dev->needs_free_netdev = true;
 	dev_net_set(dev, wiphy_net(wiphy));
 	dev->ieee80211_ptr = &vif->wdev;
-	ether_addr_copy(dev->dev_addr, vif->mac_addr);
+	eth_hw_addr_set(dev, vif->mac_addr);
 	dev->flags |= IFF_BROADCAST | IFF_MULTICAST;
 	dev->watchdog_timeo = QTNF_DEF_WDOG_TIMEOUT;
 	dev->tx_queue_len = 100;
